@@ -183,8 +183,7 @@ def _plot_stats(st, pdf, q, q_bak, detect_umi):
     _plot_pd_bars(rs.copy(), "Classification of output reads", R, ann=True)
     found, rescue, unusable = float(rs.loc[rs.Name == "Primers_found", ].Value), float(rs.loc[rs.Name == "Rescue", ].Value), float(rs.loc[rs.Name == "Unusable", ].Value)
     rs_stats = st.loc[st.Category == "ReadStats", ]
-    umi_detected = float(rs_stats.loc[rs_stats.Name == "Umi_detected", ].Value)
-    umi_detected_final = float(rs_stats.loc[rs_stats.Name == "Umi_detected_final", ].Value)
+    umi_detected_final = float(rs_stats.loc[rs_stats.Name == "Umi_detected_final", 'Value'].iloc[0])
     total = found + rescue + unusable
     found = found / total * 100
     rescue = rescue / total * 100
@@ -236,8 +235,10 @@ def main():
         '-c', metavar='config_file', type=str, default=None,
         help="File to specify primer configurations for each direction (None).")
     parser.add_argument(
-        '-k', metavar='kit', type=str, default="PCS109",
-        help="Use primer sequences from this kit (PCS109).")
+        '-k', type=str, default="PCS109",
+        help="Use primer sequences from this kit (PCS109).",
+        choices=['PCS109', 'PCS110', 'PCS111', 'LSK114']
+    )
     parser.add_argument(
         '-q', metavar='cutoff', type=float, default=None,
         help="Cutoff parameter (autotuned).")
@@ -333,7 +334,12 @@ def main():
             "HMM": os.path.join(
                 os.path.dirname(phmm_data.__file__), "PCS110_primers.hmm"),
             "FAS": os.path.join(
-                os.path.dirname(primer_data.__file__), "PCS111_primers.fas")}
+                os.path.dirname(primer_data.__file__), "PCS111_primers.fas")},
+        "LSK114": {
+            "HMM": os.path.join(
+                os.path.dirname(phmm_data.__file__), "cDNA_SSP_VNP.hmm"),
+            "FAS": os.path.join(
+                os.path.dirname(primer_data.__file__), "LSK114_primers.fas")}
     }
 
     if args.g is None:
@@ -355,13 +361,9 @@ def main():
             CONFIG = "-:VNP,-VNP"
 
     config = utils.parse_config_string(CONFIG)
-    sys.stderr.write("Using kit: {}\n".format(args.k))
+    sys.stderr.write("Using kit: {}\n".format(args.b if args.b else args.k))
     sys.stderr.write("Configurations to consider: \"{}\"\n".format(CONFIG))
 
-    in_fh = sys.stdin
-    if args.input_fastx != '-':
-        in_fh = _opener(args.input_fastx, "r")
-
     out_fh = sys.stdout
     if args.output_fastx != '-':
         out_fh = open(args.output_fastx, "w")
@@ -437,8 +439,7 @@ def main():
         sys.stderr.write(
             "Total fastq records in input file: {}\n".format(nr_records))
         read_sample = list(
-            seu.readfq(_opener(args.input_fastx, "r"), sample=target_prop,
-                       min_qual=args.Q))
+            seu.readfq(args.input_fastx, sample=target_prop, min_qual=args.Q))
         sys.stderr.write(
             "Tuning the cutoff parameter (q) on {} sampled reads ({:.1f}%) passing quality filters (Q >= {}).\n".format(
                 len(read_sample), target_prop * 100.0, args.Q))
@@ -446,15 +447,10 @@ def main():
         for qv in tqdm.tqdm(cutoffs):
             clsLen = 0
             cls = 0
-            with concurrent.futures.ProcessPoolExecutor(
-                    max_workers=args.t) as executor:
+            with concurrent.futures.ProcessPoolExecutor(max_workers=args.t) as executor:
                 for batch in utils.batch(read_sample, int((len(read_sample)))):
-                    for read, (segments, hits, usable_len) in backend(batch,
-                                                                      executor,
-                                                                      qv,
-                                                                      max(1000,
-                                                                          int((
-                                                                              len(read_sample)) / args.t))):
+                    for read, (segments, hits, usable_len) \
+                            in backend(batch, executor, qv, max(1000, int((len(read_sample)) / args.t))):
                         flt = list([x.Len for x in segments if x.Len > 0])
                         if len(flt) == 1:
                             clsLen += sum(flt)
@@ -494,7 +490,7 @@ def main():
     with concurrent.futures.ProcessPoolExecutor(
             max_workers=args.t) as executor:
         for batch in utils.batch(
-                seu.readfq(in_fh, min_qual=args.Q, rfq_sup=rfq_sup), args.B):
+                seu.readfq(args.input_fastx, min_qual=args.Q, rfq_sup=rfq_sup), args.B):
             for read, (segments, hits, usable_len) in backend(batch, executor,
                                                               q=args.q,
                                                               mb=min_batch_size):
@@ -508,9 +504,10 @@ def main():
                                                            args.p, args.y, args.U):
                     if trim_read.Umi:
                         st["Umi_detected"] += 1
-                    if args.l is not None and len(trim_read.Seq) < args.z:
+                    if len(trim_read.Seq) < args.z:
                         st["LenFail"] += 1
-                        seu.writefq(trim_read, l_fh)
+                        if args.l is not None:
+                            seu.writefq(trim_read, l_fh)
                         continue
                     if len(segments) == 1:
                         if trim_read.Umi:
@@ -546,7 +543,7 @@ def main():
     if args.S is not None:
         stdf.to_csv(args.S, sep="\t", index=False)
 
-    for fh in (in_fh, out_fh, u_fh, l_fh, w_fh, a_fh, d_fh):
+    for fh in (out_fh, u_fh, l_fh, w_fh, a_fh, d_fh):
         if fh is None:
             continue
         fh.flush()
